@@ -45,8 +45,7 @@
  * An implementation of #PolkitBackendAuthority that stores
  * authorizations on the local file system, supports interaction with
  * authentication agents (virtue of being based on
- * #PolkitBackendInteractiveAuthority), and implements support for
- * lock down.
+ * #PolkitBackendInteractiveAuthority).
  */
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -56,8 +55,6 @@ static GList *get_users_in_group (PolkitIdentity              *group,
 
 static GList *get_groups_for_user (PolkitIdentity              *user);
 
-static void register_extensions (void);
-
 /* ---------------------------------------------------------------------------------------------------- */
 
 typedef struct
@@ -66,6 +63,9 @@ typedef struct
 
   GList *authorization_stores;
 
+  GFileMonitor *sysconf_dir_monitor;
+  GFileMonitor *localstate_dir_monitor;
+
 } PolkitBackendLocalAuthorityPrivate;
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -89,27 +89,6 @@ static PolkitImplicitAuthorization polkit_backend_local_authority_check_authoriz
                                                           PolkitImplicitAuthorization        implicit,
                                                           PolkitDetails                     *out_details);
 
-static void polkit_backend_local_authority_add_lockdown_for_action (PolkitBackendAuthority  *authority,
-                                                                    PolkitSubject           *caller,
-                                                                    const gchar             *action_id,
-                                                                    GAsyncReadyCallback      callback,
-                                                                    gpointer                 user_data);
-
-static gboolean polkit_backend_local_authority_add_lockdown_for_action_finish (PolkitBackendAuthority  *authority,
-                                                                               GAsyncResult            *res,
-                                                                               GError                 **error);
-
-static void polkit_backend_local_authority_remove_lockdown_for_action (PolkitBackendAuthority  *authority,
-                                                                       PolkitSubject           *caller,
-                                                                       const gchar             *action_id,
-                                                                       GAsyncReadyCallback      callback,
-                                                                       gpointer                 user_data);
-
-static gboolean polkit_backend_local_authority_remove_lockdown_for_action_finish (PolkitBackendAuthority  *authority,
-                                                                                  GAsyncResult            *res,
-                                                                                  GError                 **error);
-
-
 G_DEFINE_TYPE_WITH_CODE (PolkitBackendLocalAuthority,
                          polkit_backend_local_authority,
                          POLKIT_BACKEND_TYPE_INTERACTIVE_AUTHORITY,
@@ -131,43 +110,226 @@ on_store_changed (PolkitBackendLocalAuthorizationStore *store,
   g_signal_emit_by_name (authority, "changed");
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+purge_all_authorization_stores (PolkitBackendLocalAuthority *authority)
+{
+  PolkitBackendLocalAuthorityPrivate *priv;
+  GList *l;
+
+  priv = POLKIT_BACKEND_LOCAL_AUTHORITY_GET_PRIVATE (authority);
+
+  for (l = priv->authorization_stores; l != NULL; l = l->next)
+    {
+      PolkitBackendLocalAuthorizationStore *store = POLKIT_BACKEND_LOCAL_AUTHORIZATION_STORE (l->data);
+      g_signal_handlers_disconnect_by_func (store,
+                                            G_CALLBACK (on_store_changed),
+                                            authority);
+      g_object_unref (store);
+    }
+  g_list_free (priv->authorization_stores);
+  priv->authorization_stores = NULL;
+
+  g_debug ("Purged all local authorization stores");
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+add_one_authorization_store (PolkitBackendLocalAuthority *authority,
+                             GFile                       *directory)
+{
+  PolkitBackendLocalAuthorizationStore *store;
+  PolkitBackendLocalAuthorityPrivate *priv;
+
+  priv = POLKIT_BACKEND_LOCAL_AUTHORITY_GET_PRIVATE (authority);
+
+  store = polkit_backend_local_authorization_store_new (directory, ".pkla");
+  priv->authorization_stores = g_list_append (priv->authorization_stores, store);
+
+  g_signal_connect (store,
+                    "changed",
+                    G_CALLBACK (on_store_changed),
+                    authority);
+}
+
+static gint
+authorization_store_path_compare_func (GFile *file_a,
+                                       GFile *file_b)
+{
+  const gchar *a;
+  const gchar *b;
+
+  a = g_object_get_data (G_OBJECT (file_a), "sort-key");
+  b = g_object_get_data (G_OBJECT (file_b), "sort-key");
+
+  return g_strcmp0 (a, b);
+}
+
+static void
+add_all_authorization_stores (PolkitBackendLocalAuthority *authority)
+{
+  guint n;
+  GList *directories;
+  GList *l;
+
+  directories = NULL;
+
+  for (n = 0; n < 2; n++)
+    {
+      const gchar *toplevel_path;
+      GFile *toplevel_directory;
+      GFileEnumerator *directory_enumerator;
+      GFileInfo *file_info;
+      GError *error;
+
+      error = NULL;
+
+      if (n == 0)
+        toplevel_path = PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority";
+      else
+        toplevel_path = PACKAGE_SYSCONF_DIR "/polkit-1/localauthority";
+
+      toplevel_directory = g_file_new_for_path (toplevel_path);
+      directory_enumerator = g_file_enumerate_children (toplevel_directory,
+                                                        "standard::*",
+                                                        G_FILE_QUERY_INFO_NONE,
+                                                        NULL,
+                                                        &error);
+      if (directory_enumerator == NULL)
+        {
+          g_warning ("Error getting enumerator for %s: %s", toplevel_path, error->message);
+          g_error_free (error);
+          g_object_unref (toplevel_directory);
+          continue;
+        }
+
+      while ((file_info = g_file_enumerator_next_file (directory_enumerator, NULL, &error)) != NULL)
+        {
+          /* only consider directories */
+          if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY)
+            {
+              const gchar *name;
+              GFile *directory;
+              gchar *sort_key;
+
+              name = g_file_info_get_name (file_info);
+
+              /* This makes entries in directories in /etc take precedence to entries in directories in /var */
+              sort_key = g_strdup_printf ("%s-%d", name, n);
+
+              directory = g_file_get_child (toplevel_directory, name);
+              g_object_set_data_full (G_OBJECT (directory), "sort-key", sort_key, g_free);
+
+              directories = g_list_prepend (directories, directory);
+            }
+          g_object_unref (file_info);
+        }
+      if (error != NULL)
+        {
+          g_warning ("Error enumerating files in %s: %s", toplevel_path, error->message);
+          g_error_free (error);
+          g_object_unref (toplevel_directory);
+          g_object_unref (directory_enumerator);
+          continue;
+        }
+      g_object_unref (directory_enumerator);
+      g_object_unref (toplevel_directory);
+    }
+
+  /* Sort directories */
+  directories = g_list_sort (directories, (GCompareFunc) authorization_store_path_compare_func);
+
+  /* And now add an authorization store for each one */
+  for (l = directories; l != NULL; l = l->next)
+    {
+      GFile *directory = G_FILE (l->data);
+      gchar *name;
+
+      name = g_file_get_path (directory);
+      g_debug ("Added `%s' as a local authorization store", name);
+      g_free (name);
+
+      add_one_authorization_store (authority, directory);
+    }
+
+  g_list_foreach (directories, (GFunc) g_object_unref, NULL);
+  g_list_free (directories);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+on_toplevel_authority_store_monitor_changed (GFileMonitor     *monitor,
+                                             GFile            *file,
+                                             GFile            *other_file,
+                                             GFileMonitorEvent event_type,
+                                             gpointer          user_data)
+{
+  PolkitBackendLocalAuthority *authority = POLKIT_BACKEND_LOCAL_AUTHORITY (user_data);
+
+  purge_all_authorization_stores (authority);
+  add_all_authorization_stores (authority);
+}
+
 static void
 polkit_backend_local_authority_init (PolkitBackendLocalAuthority *authority)
 {
   PolkitBackendLocalAuthorityPrivate *priv;
-  GFile *directory;
+  GFile *config_directory;
   guint n;
-  const gchar *store_locations[] =
-    {
-      PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority/10-vendor.d",
-      PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority/20-org.d",
-      PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority/30-site.d",
-      PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority/50-local.d",
-      PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority/90-mandatory.d",
-      NULL
-    };
 
   priv = POLKIT_BACKEND_LOCAL_AUTHORITY_GET_PRIVATE (authority);
 
-  directory = g_file_new_for_path (PACKAGE_SYSCONF_DIR "/polkit-1/localauthority.conf.d");
-  priv->config_source = polkit_backend_config_source_new (directory);
-  g_object_unref (directory);
+  config_directory = g_file_new_for_path (PACKAGE_SYSCONF_DIR "/polkit-1/localauthority.conf.d");
+  priv->config_source = polkit_backend_config_source_new (config_directory);
+  g_object_unref (config_directory);
 
-  for (n = 0; store_locations[n] != NULL; n++)
+  add_all_authorization_stores (authority);
+
+  /* Monitor the toplevels */
+  for (n = 0; n < 2; n++)
     {
-      PolkitBackendLocalAuthorizationStore *store;
+      const gchar *toplevel_path;
+      GFile *toplevel_directory;
+      GFileMonitor *monitor;
+      GError *error;
 
-      directory = g_file_new_for_path (store_locations[n]);
-      store = polkit_backend_local_authorization_store_new (directory, ".pkla");
-      priv->authorization_stores = g_list_prepend (priv->authorization_stores, store);
-      g_object_unref (directory);
+      if (n == 0)
+        toplevel_path = PACKAGE_LOCALSTATE_DIR "/lib/polkit-1/localauthority";
+      else
+        toplevel_path = PACKAGE_SYSCONF_DIR "/polkit-1/localauthority";
 
-      g_signal_connect (store,
+      toplevel_directory = g_file_new_for_path (toplevel_path);
+
+      error = NULL;
+      monitor = g_file_monitor_directory (toplevel_directory,
+                                          G_FILE_MONITOR_NONE,
+                                          NULL,
+                                          &error);
+      if (monitor == NULL)
+        {
+          g_warning ("Error creating file monitor for %s: %s", toplevel_path, error->message);
+          g_error_free (error);
+          g_object_unref (toplevel_directory);
+          continue;
+        }
+
+      g_debug ("Monitoring `%s' for changes", toplevel_path);
+
+      g_signal_connect (monitor,
                         "changed",
-                        G_CALLBACK (on_store_changed),
+                        G_CALLBACK (on_toplevel_authority_store_monitor_changed),
                         authority);
+
+      if (n == 0)
+        priv->sysconf_dir_monitor = monitor;
+      else
+        priv->localstate_dir_monitor = monitor;
+
+      g_object_unref (toplevel_directory);
     }
-  priv->authorization_stores = g_list_reverse (priv->authorization_stores);
 }
 
 static void
@@ -179,12 +341,16 @@ polkit_backend_local_authority_finalize (GObject *object)
   local_authority = POLKIT_BACKEND_LOCAL_AUTHORITY (object);
   priv = POLKIT_BACKEND_LOCAL_AUTHORITY_GET_PRIVATE (local_authority);
 
+  purge_all_authorization_stores (local_authority);
+
+  if (priv->sysconf_dir_monitor != NULL)
+    g_object_unref (priv->sysconf_dir_monitor);
+  if (priv->localstate_dir_monitor != NULL)
+    g_object_unref (priv->localstate_dir_monitor);
+
   if (priv->config_source != NULL)
     g_object_unref (priv->config_source);
 
-  g_list_foreach (priv->authorization_stores, (GFunc) g_object_unref, NULL);
-  g_list_free (priv->authorization_stores);
-
   G_OBJECT_CLASS (polkit_backend_local_authority_parent_class)->finalize (object);
 }
 
@@ -203,7 +369,7 @@ polkit_backend_local_authority_get_version (PolkitBackendAuthority *authority)
 static PolkitAuthorityFeatures
 polkit_backend_local_authority_get_features (PolkitBackendAuthority *authority)
 {
-  return POLKIT_AUTHORITY_FEATURES_TEMPORARY_AUTHORIZATION | POLKIT_AUTHORITY_FEATURES_LOCKDOWN;
+  return POLKIT_AUTHORITY_FEATURES_TEMPORARY_AUTHORIZATION;
 }
 
 static void
@@ -221,16 +387,10 @@ polkit_backend_local_authority_class_init (PolkitBackendLocalAuthorityClass *kla
   authority_class->get_name                             = polkit_backend_local_authority_get_name;
   authority_class->get_version                          = polkit_backend_local_authority_get_version;
   authority_class->get_features                         = polkit_backend_local_authority_get_features;
-  authority_class->add_lockdown_for_action              = polkit_backend_local_authority_add_lockdown_for_action;
-  authority_class->add_lockdown_for_action_finish       = polkit_backend_local_authority_add_lockdown_for_action_finish;
-  authority_class->remove_lockdown_for_action           = polkit_backend_local_authority_remove_lockdown_for_action;
-  authority_class->remove_lockdown_for_action_finish    = polkit_backend_local_authority_remove_lockdown_for_action_finish;
   interactive_authority_class->get_admin_identities     = polkit_backend_local_authority_get_admin_auth_identities;
   interactive_authority_class->check_authorization_sync = polkit_backend_local_authority_check_authorization_sync;
 
   g_type_class_add_private (klass, sizeof (PolkitBackendLocalAuthorityPrivate));
-
-  register_extensions ();
 }
 
 static GList *
@@ -503,469 +663,3 @@ get_groups_for_user (PolkitIdentity *user)
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
-
-static gchar *
-lockdown_get_filename (const gchar *action_id)
-{
-  return g_strdup_printf (PACKAGE_LOCALSTATE_DIR
-                          "/lib/polkit-1/localauthority/90-mandatory.d/"
-                          "org.freedesktop.policykit.localauthority.lockdown.action-%s.pkla",
-                          action_id);
-}
-
-static gboolean
-lockdown_exists (const gchar *action_id)
-{
-  gchar *filename;
-  gboolean ret;
-
-  ret = FALSE;
-
-  filename = lockdown_get_filename (action_id);
-  if (g_file_test (filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS))
-    ret = TRUE;
-  g_free (filename);
-
-  return ret;
-}
-
-static gboolean
-lockdown_add (const gchar  *action_id,
-              GError      **error)
-{
-  gboolean ret;
-  gchar *filename;
-  gchar *contents;
-
-  ret = FALSE;
-
-  filename = lockdown_get_filename (action_id);
-  contents = g_strdup_printf ("# Added by pklalockdown(1)\n"
-                              "#\n"
-                              "[Lockdown]\n"
-                              "Identity=unix-user:*\n"
-                              "Action=%s\n"
-                              "ResultAny=no\n"
-                              "ResultInactive=no\n"
-                              "ResultActive=auth_admin_keep\n"
-                              "ReturnValue=polkit.lockdown=1",
-                              action_id);
-  if (!g_file_set_contents (filename,
-                            contents,
-                            -1,
-                            error))
-    goto out;
-
-  ret = TRUE;
-
- out:
-  g_free (filename);
-  g_free (contents);
-  return ret;
-}
-
-static gboolean
-lockdown_remove (const gchar  *action_id,
-                 GError      **error)
-{
-  gboolean ret;
-  gchar *filename;
-
-  ret = FALSE;
-
-  filename = lockdown_get_filename (action_id);
-  if (g_unlink (filename) != 0)
-    {
-      g_set_error (error,
-                   POLKIT_ERROR,
-                   POLKIT_ERROR_FAILED,
-                   "Cannot unlink file %s: %s\n",
-                   filename,
-                   g_strerror (errno));
-      goto out;
-    }
-
-  ret = TRUE;
-
- out:
-  g_free (filename);
-  return ret;
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-add_lockdown_check_auth_cb (PolkitBackendAuthority *authority,
-                            GAsyncResult           *res,
-                            gpointer                user_data)
-{
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
-  PolkitAuthorizationResult *result;
-  GError *error;
-
-  result = polkit_backend_authority_check_authorization_finish (authority,
-                                                                res,
-                                                                &error);
-  if (result == NULL)
-    {
-      g_simple_async_result_set_from_error (simple, error);
-      g_error_free (error);
-    }
-  else
-    {
-      if (polkit_authorization_result_get_is_authorized (result))
-        {
-          const gchar *action_id;
-
-          action_id = g_object_get_data (G_OBJECT (simple), "lock-down-action-id");
-
-          if (lockdown_exists (action_id))
-            {
-              g_simple_async_result_set_error (simple,
-                                               POLKIT_ERROR,
-                                               POLKIT_ERROR_FAILED,
-                                               "Action %s is already locked down",
-                                               action_id);
-            }
-          else
-            {
-              GError *error;
-
-              error = NULL;
-              if (!lockdown_add (action_id, &error))
-                {
-                  g_simple_async_result_set_error (simple,
-                                                   POLKIT_ERROR,
-                                                   POLKIT_ERROR_FAILED,
-                                                   "Error adding lock down for action %s: %s",
-                                                   action_id,
-                                                   error->message);
-                  g_error_free (error);
-                }
-            }
-        }
-      else
-        {
-          g_simple_async_result_set_error (simple,
-                                           POLKIT_ERROR,
-                                           POLKIT_ERROR_NOT_AUTHORIZED,
-                                           "Not authorized to add lock down for the requested action");
-        }
-      g_object_unref (result);
-    }
-
-  g_simple_async_result_complete (simple);
-  g_object_unref (simple);
-}
-
-static void
-polkit_backend_local_authority_add_lockdown_for_action (PolkitBackendAuthority  *authority,
-                                                        PolkitSubject           *caller,
-                                                        const gchar             *action_id,
-                                                        GAsyncReadyCallback      callback,
-                                                        gpointer                 user_data)
-{
-  GSimpleAsyncResult *simple;
-  PolkitDetails *details;
-  GCancellable *cancellable;
-
-  simple = g_simple_async_result_new (G_OBJECT (authority),
-                                      callback,
-                                      user_data,
-                                      polkit_backend_local_authority_add_lockdown_for_action);
-
-  g_object_set_data_full (G_OBJECT (simple), "lock-down-action-id", g_strdup (action_id), g_free);
-
-  details = polkit_details_new ();
-  polkit_details_insert (details, "action-id", action_id);
-  polkit_details_insert (details, "add-lockdown", "1");
-
-  cancellable = g_cancellable_new ();
-
-  /* first check if caller is authorized for this */
-  polkit_backend_authority_check_authorization (POLKIT_BACKEND_AUTHORITY (authority),
-                                                NULL,
-                                                caller,
-                                                "org.freedesktop.policykit.lockdown",
-                                                details,
-                                                POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
-                                                cancellable,
-                                                (GAsyncReadyCallback) add_lockdown_check_auth_cb,
-                                                simple);
-
-  g_object_unref (details);
-  g_object_unref (cancellable);
-}
-
-static gboolean
-polkit_backend_local_authority_add_lockdown_for_action_finish (PolkitBackendAuthority  *authority,
-                                                               GAsyncResult            *res,
-                                                               GError                 **error)
-{
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
-
-  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == polkit_backend_local_authority_add_lockdown_for_action);
-
-  if (g_simple_async_result_propagate_error (simple, error))
-    return FALSE;
-
-  return TRUE;
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-remove_lockdown_check_auth_cb (PolkitBackendAuthority *authority,
-                            GAsyncResult           *res,
-                            gpointer                user_data)
-{
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
-  PolkitAuthorizationResult *result;
-  GError *error;
-
-  result = polkit_backend_authority_check_authorization_finish (authority,
-                                                                res,
-                                                                &error);
-  if (result == NULL)
-    {
-      g_simple_async_result_set_from_error (simple, error);
-      g_error_free (error);
-    }
-  else
-    {
-      if (polkit_authorization_result_get_is_authorized (result))
-        {
-          const gchar *action_id;
-
-          action_id = g_object_get_data (G_OBJECT (simple), "lock-down-action-id");
-
-          if (!lockdown_exists (action_id))
-            {
-              g_simple_async_result_set_error (simple,
-                                               POLKIT_ERROR,
-                                               POLKIT_ERROR_FAILED,
-                                               "Action %s is not locked down",
-                                               action_id);
-            }
-          else
-            {
-              GError *error;
-
-              error = NULL;
-              if (!lockdown_remove (action_id, &error))
-                {
-                  g_simple_async_result_set_error (simple,
-                                                   POLKIT_ERROR,
-                                                   POLKIT_ERROR_FAILED,
-                                                   "Error removing lock down for action %s: %s",
-                                                   action_id,
-                                                   error->message);
-                  g_error_free (error);
-                }
-            }
-        }
-      else
-        {
-          g_simple_async_result_set_error (simple,
-                                           POLKIT_ERROR,
-                                           POLKIT_ERROR_NOT_AUTHORIZED,
-                                           "Not authorized to remove lock down for the requested action");
-        }
-      g_object_unref (result);
-    }
-
-  g_simple_async_result_complete (simple);
-  g_object_unref (simple);
-}
-
-static void
-polkit_backend_local_authority_remove_lockdown_for_action (PolkitBackendAuthority  *authority,
-                                                        PolkitSubject           *caller,
-                                                        const gchar             *action_id,
-                                                        GAsyncReadyCallback      callback,
-                                                        gpointer                 user_data)
-{
-  GSimpleAsyncResult *simple;
-  PolkitDetails *details;
-  GCancellable *cancellable;
-
-  simple = g_simple_async_result_new (G_OBJECT (authority),
-                                      callback,
-                                      user_data,
-                                      polkit_backend_local_authority_remove_lockdown_for_action);
-
-  g_object_set_data_full (G_OBJECT (simple), "lock-down-action-id", g_strdup (action_id), g_free);
-
-  details = polkit_details_new ();
-  polkit_details_insert (details, "action-id", action_id);
-  polkit_details_insert (details, "remove-lockdown", "1");
-
-  cancellable = g_cancellable_new ();
-
-  /* first check if caller is authorized for this */
-  polkit_backend_authority_check_authorization (POLKIT_BACKEND_AUTHORITY (authority),
-                                                NULL,
-                                                caller,
-                                                "org.freedesktop.policykit.lockdown",
-                                                details,
-                                                POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
-                                                cancellable,
-                                                (GAsyncReadyCallback) remove_lockdown_check_auth_cb,
-                                                simple);
-
-  g_object_unref (details);
-  g_object_unref (cancellable);
-}
-
-static gboolean
-polkit_backend_local_authority_remove_lockdown_for_action_finish (PolkitBackendAuthority  *authority,
-                                                               GAsyncResult            *res,
-                                                               GError                 **error)
-{
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
-
-  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == polkit_backend_local_authority_remove_lockdown_for_action);
-
-  if (g_simple_async_result_propagate_error (simple, error))
-    return FALSE;
-
-  return TRUE;
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-#define PBLA_TYPE_ACTION_LOOKUP          (pbla_action_lookup_get_type())
-#define PBLA_ACTION_LOOKUP(o)            (G_TYPE_CHECK_INSTANCE_CAST ((o), PBLA_TYPE_ACTION_LOOKUP, PblaActionLookup))
-#define PBLA_ACTION_LOOKUP_CLASS(k)      (G_TYPE_CHECK_CLASS_CAST((k), PBLA_TYPE_ACTION_LOOKUP, PblaActionLookupClass))
-#define PBLA_ACTION_LOOKUP_GET_CLASS(o)  (G_TYPE_INSTANCE_GET_CLASS ((o), PBLA_TYPE_ACTION_LOOKUP, PblaActionLookupClass))
-#define PBLA_IS_ACTION_LOOKUP(o)         (G_TYPE_CHECK_INSTANCE_TYPE ((o), PBLA_TYPE_ACTION_LOOKUP))
-#define PBLA_IS_ACTION_LOOKUP_CLASS(k)   (G_TYPE_CHECK_CLASS_TYPE ((k), PBLA_TYPE_ACTION_LOOKUP))
-
-typedef struct _PblaActionLookup PblaActionLookup;
-typedef struct _PblaActionLookupClass PblaActionLookupClass;
-
-struct _PblaActionLookup
-{
-  GObject parent;
-};
-
-struct _PblaActionLookupClass
-{
-  GObjectClass parent_class;
-};
-
-GType pbla_action_lookup_get_type (void) G_GNUC_CONST;
-
-static void pbla_action_lookup_iface_init (PolkitBackendActionLookupIface *iface);
-
-
-G_DEFINE_TYPE_EXTENDED (PblaActionLookup,
-                        pbla_action_lookup,
-                        G_TYPE_OBJECT,
-                        0,
-                        G_IMPLEMENT_INTERFACE (POLKIT_BACKEND_TYPE_ACTION_LOOKUP,
-                                               pbla_action_lookup_iface_init))
-
-static void
-pbla_action_lookup_init (PblaActionLookup *lookup)
-{
-}
-
-static void
-pbla_action_lookup_class_init (PblaActionLookupClass *klass)
-{
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-static gchar *
-pbla_action_lookup_get_message   (PolkitBackendActionLookup *lookup,
-                                  const gchar               *action_id,
-                                  PolkitDetails             *details,
-                                  PolkitActionDescription   *action_description)
-{
-  gchar *ret;
-  const gchar *s;
-
-  ret = NULL;
-
-  if (g_strcmp0 (action_id, "org.freedesktop.policykit.lockdown") != 0)
-    goto out;
-
-  s = polkit_details_lookup (details, "remove-lockdown");
-  if (s == NULL)
-    {
-      ret = g_strdup (_("Authentication is needed to lock down an action"));
-    }
-  else
-    {
-      ret = g_strdup (_("Authentication is needed to remove lock down for an action"));
-    }
-
- out:
-  return ret;
-}
-
-static gchar *
-pbla_action_lookup_get_icon_name (PolkitBackendActionLookup *lookup,
-                                  const gchar               *action_id,
-                                  PolkitDetails             *details,
-                                  PolkitActionDescription   *action_description)
-{
-  gchar *ret;
-
-  ret = NULL;
-
-  /* explicitly left blank for now */
-
-  return ret;
-}
-
-static PolkitDetails *
-pbla_action_lookup_get_details (PolkitBackendActionLookup *lookup,
-                                const gchar               *action_id,
-                                PolkitDetails             *details,
-                                PolkitActionDescription   *action_desc)
-{
-  PolkitDetails *ret;
-  const gchar *s;
-  const gchar *s2;
-
-  ret = NULL;
-
-  if (g_strcmp0 (action_id, "org.freedesktop.policykit.lockdown") != 0)
-    goto out;
-
-  s = polkit_details_lookup (details, "action-id");
-  if (s == NULL)
-    goto out;
-
-  s2 = polkit_details_lookup (details, "remove-lockdown");
-
-  ret = polkit_details_new ();
-  if (s2 == NULL)
-    polkit_details_insert (ret, _("Action to lock down"), s);
-  else
-    polkit_details_insert (ret, _("Locked down action"), s);
-
- out:
-  return ret;
-}
-
-static void
-pbla_action_lookup_iface_init (PolkitBackendActionLookupIface *iface)
-{
-  iface->get_message   = pbla_action_lookup_get_message;
-  iface->get_icon_name = pbla_action_lookup_get_icon_name;
-  iface->get_details   = pbla_action_lookup_get_details;
-}
-
-
-static void
-register_extensions (void)
-{
-  g_io_extension_point_implement (POLKIT_BACKEND_ACTION_LOOKUP_EXTENSION_POINT_NAME,
-                                  PBLA_TYPE_ACTION_LOOKUP,
-                                  "lockdown action lookup extension " PACKAGE_VERSION,
-                                  0);
-}
